@@ -1,19 +1,19 @@
 import os
-import datetime
 import re
 from dataclasses import dataclass, field
 from itertools import takewhile
-from typing import Dict, Iterator, List
+from typing import Dict, Iterator, List, Tuple
 
 from pheasant.core.decorator import commentable, surround
 from pheasant.core.renderer import Renderer
+from pheasant.renderers.jupyter.filters import get_metadata
 from pheasant.renderers.jupyter.ipython import (extra_html, get_extra_module,
                                                 latex_display_format,
                                                 select_display_data,
+                                                select_last_display_data,
                                                 select_outputs)
 from pheasant.renderers.jupyter.kernel import (format_report, kernels,
                                                output_hook)
-from pheasant.utils.cache import delete_cache, load_cache, save_cache
 from pheasant.utils.progress import ProgressBar, progress_bar_factory
 
 
@@ -33,14 +33,11 @@ class CacheMismatchError(BaseException):
 
 
 class Jupyter(Renderer):
-    option: str = field(default="", init=False)
+    language: str = "python"
     count: int = field(default=0, init=False)
     cache: List[Cell] = field(default_factory=list, init=False)
     extra_html: str = field(default="", init=False)
     progress_bar: ProgressBar = field(default_factory=progress_bar_factory, init=False)
-    enabled: bool = field(default=True, init=False)
-    safe: bool = field(default=False, init=False)  # If True, code must match cache.
-    verbose: int = 1
 
     FENCED_CODE_PATTERN = (
         r"^(?P<mark>`{3,})(?P<language>\w*) ?(?P<option>.*?)\n"
@@ -52,12 +49,16 @@ class Jupyter(Renderer):
     def init(self):
         self.register(Jupyter.FENCED_CODE_PATTERN, self.render_fenced_code)
         self.register(Jupyter.INLINE_CODE_PATTERN, self.render_inline_code)
-        self.set_template(["fenced_code", "inline_code"])
+        templates = self.set_template(["fenced_code", "inline_code"])
+        templates[0].environment.filters["get_metadata"] = get_metadata
+        # safe: If True, code must match cache.
+        # verbose: 0: no info, 1: output, 2: code and output
+        self.set_config(enabled=True, safe=False, verbose=0)
 
     def enter(self):
         self.count = 0
         self.progress_bar.total = len(self.findall())
-        self.cache, self.extra_html = load_cache(self.page.path) or ([], "")
+        self.cache, self.extra_html = self.page.cache.load() or ([], "")
 
     def exit(self):
         self.progress_bar.finish(count=self.count)
@@ -66,10 +67,10 @@ class Jupyter(Renderer):
             self.extra_html = extra_html(extra_modules)
         self.page.meta["extra_html"] = self.extra_html
 
-        if self.enabled and self.page.path and self.cache:
+        if self.config["enabled"] and self.page.path and self.cache:
             for cell in self.cache:
                 cell.cached = True
-            save_cache(self.page.path, (self.cache, self.extra_html))
+            self.page.cache.save((self.cache, self.extra_html))
 
     def get_extra_modules(self) -> Iterator[str]:
         for cell in self.cache:
@@ -80,30 +81,28 @@ class Jupyter(Renderer):
                 break
 
     def render_fenced_code(self, context, splitter, parser) -> Iterator[str]:
+        context["code"] = context["code"].replace("  # isort:skip", "")
+        context["code"] = context["code"].replace("  # type:ignore", "")
         code = context["code"]
         if code.startswith("# option:"):
             index = code.index("\n")
-            option = code[9:index].strip()
-            if context["option"]:
-                context["option"] = " ".join([context["option"], option])
-            else:
-                context["option"] = option
-            code = code[index + 1 :]
+            option, code = code[9:index], code[index + 1 :]
+            context["option"] = " ".join([option, context["option"]]).strip()
             context["code"] = code
         if "inline" in context["option"]:
-            self.option = context["option"] + " fenced-code"
-            splitter.send("{{" + code + "}}")
-            self.option = ""
+            option = context["option"] + " fenced-code"
+            splitter.send("{{" + code + "#" + option + "}}")
             return
-        if "debug" in context["option"]:
-            context["code"] = code
         yield "\n" + self.execute_and_render(code, context, "fenced_code") + "\n\n"
 
     @commentable("code")
     def render_inline_code(self, context, splitter, parser) -> Iterator[str]:
-        context["option"] = self.option
-        code = context["code"]
-        if "fenced-code" not in self.option:
+        code, context["option"] = split_option(context["code"])
+        if "inspect" in context["option"]:
+            source = f"\n```python inspect hide-input\n{code}\n```\n"
+            splitter.send(source)
+            return
+        elif "fenced-code" not in context["option"]:
             code = code.replace(";", "\n")
         yield self.execute_and_render(code, context, "inline_code")
 
@@ -111,30 +110,30 @@ class Jupyter(Renderer):
         self.count += 1
 
         cell = Cell(code, context, template)
-        if len(self.cache) >= self.count and "run" not in context["option"]:
+        if len(self.cache) >= self.count:
             cached = self.cache[self.count - 1]
-            if cell == cached:
-                if (self.count - 1) % 5 == 0 and self.page.path:
+            if "freeze" in context["option"] or cell == cached:
+                if self.page.path and (self.count - 1) % 5 == 0:
                     relpath = os.path.relpath(self.page.path)
                     self.progress_bar.progress(relpath, count=self.count)
                 return surround(cached.output, "cached")
-            elif self.safe and self.page.path:
-                delete_cache(self.page.path)
+            elif self.page.path and self.config["safe"]:
+                self.page.cache.delete()
                 self.progress_bar.finish(done=False)
                 raise CacheMismatchError
 
-        if not self.enabled:
-            return self.render(
-                template, context, outputs=[], report={"count": self.count}
-            )
+        if not self.config["enabled"]:
+            report = {"count": self.count}
+            return self.render(template, context, outputs=[], report=report)
 
-        language = context.get("language", kernels.language)
-        kernel_name = kernels.get_kernel_name(language)
+        self.language = context.get("language", self.language)
+        kernel_name = kernels.get_kernel_name(self.language)
 
         if not kernel_name:
-            return self.render(
-                template, context, outputs=[], report={"count": self.count}
-            )
+            report = {"count": self.count}
+            cell.output = self.render(template, context, outputs=[], report=report)
+            self.update_cache(cell)
+            return cell.output
 
         kernel = kernels.get_kernel(kernel_name)
         kernel.start(silent=self.page.path == "")
@@ -142,27 +141,52 @@ class Jupyter(Renderer):
         if self.count == 1:
             self.progress_bar.progress("Start", count=self.count)
 
+        kwargs = ""
+        if self.language == "python":
+            _, kwargs = split_kwargs_from_option(context["option"])
+            if kwargs:
+                kernel.execute(
+                    "from pheasant.renderers.jupyter.ipython import formatter_kwargs\n"
+                    f"formatter_kwargs.update(dict({kwargs}))"
+                )
+
+        verbose = self.config["verbose"]
+
         def execute():
-            outputs = kernel.execute(
-                code, output_hook=output_hook if self.verbose else None
-            )
+            if verbose >= 2:
+                codes = [self.language + "> " + line for line in code.split("\n")]
+                print("\n".join(codes))
+            func = kernel.inspect if "inspect" in context["option"] else kernel.execute
+            try:
+                outputs = func(code, output_hook=output_hook if verbose else None)
+            except NameError:
+                if self.page.path:
+                    self.page.cache.delete()
+                    self.progress_bar.finish(done=False)
+                raise NameError(f"Cell number: {self.count}\n{context['code']}")
             report = format_report(kernel.report)
             report["count"] = self.count
             return outputs, report
 
         def format(result):
             relpath = os.path.relpath(self.page.path)
-            datetime_format = r"%H:%M:%S"
-            now = datetime.datetime.now().strftime(datetime_format)
-            return f"{relpath} ({result[1]['total']}) {now}"
+            return f"{relpath}({result[1]['total']})"
 
         outputs, report = self.progress_bar.progress(execute, format, self.count)
 
+        if kwargs:
+            kernel.execute(
+                "from pheasant.renderers.jupyter.ipython import formatter_kwargs\n"
+                "formatter_kwargs.clear()"
+            )
+
         cell.extra_module = get_extra_module(outputs)
         select_display_data(outputs)
 
         if "debug" in context["option"]:
             outputs = [{"type": "execute_result", "data": {"text/plain": outputs}}]
+        elif "display-last" in context["option"]:
+            select_last_display_data(outputs)
         elif template == "inline_code":
             select_outputs(outputs)
         else:
@@ -172,14 +196,49 @@ class Jupyter(Renderer):
             return output["type"] != "error" or output["ename"] != "SystemExit"
 
         outputs = list(takewhile(not_system_exit, outputs))
+        option = context["option"].split()
+        code = context["code"].replace("\n\n\n", "\n\n")
         cell.output = self.render(
-            template, context, kernel_name=kernel_name, outputs=outputs, report=report
+            template,
+            context,
+            code=code,
+            option=option,
+            kernel_name=kernel_name,
+            outputs=outputs,
+            report=report,
         )
+        self.update_cache(cell)
+        return cell.output
 
+    def update_cache(self, cell: Cell) -> None:
         if len(self.cache) == self.count - 1:
             self.cache.append(cell)
-        else:
-            self.cache[self.count - 1] = cell
-            if len(self.cache) > self.count:
+            return
+
+        self.cache[self.count - 1] = cell
+        if len(self.cache) > self.count:
+            if "freeze" not in cell.context["option"]:
                 self.cache[self.count].valid = False
-        return cell.output
+
+
+def split_option(code: str) -> Tuple[str, str]:
+    if "#" not in code:
+        return code, ""
+    code, option = code.split("#")
+    return code.strip(), option.strip()
+
+
+# TODO: kwargs which contains space.
+def split_kwargs_from_option(option: str) -> Tuple[str, str]:
+    if "=" not in option:
+        return option, ""
+    options = option.split(" ")
+    option_list = []
+    kwargs_list = []
+    for x in options:
+        if "=" in x:
+            kwargs_list.append(x)
+        elif x:
+            option_list.append(x)
+
+    return " ".join(option_list), ",".join(kwargs_list)
